package bruhcollective.itaysonlab.jetispot.ui.screens.hub

import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.GridItemSpan
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.foundation.lazy.grid.items
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Error
import androidx.compose.material3.CircularProgressIndicator
import androidx.compose.material3.Icon
import androidx.compose.material3.OutlinedButton
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.ViewModel
import androidx.navigation.NavController
import bruhcollective.itaysonlab.jetispot.core.SpApiManager
import bruhcollective.itaysonlab.jetispot.core.SpPlayerServiceManager
import bruhcollective.itaysonlab.jetispot.core.api.SpInternalApi
import bruhcollective.itaysonlab.jetispot.core.objs.hub.HubResponse
import bruhcollective.itaysonlab.jetispot.core.objs.hub.isGrid
import bruhcollective.itaysonlab.jetispot.core.objs.player.PlayFromContextData
import bruhcollective.itaysonlab.jetispot.ui.hub.HubBinder
import bruhcollective.itaysonlab.jetispot.ui.hub.HubScreenDelegate
import bruhcollective.itaysonlab.jetispot.ui.shared.PagingErrorPage
import bruhcollective.itaysonlab.jetispot.ui.shared.PagingLoadingPage
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.launch
import javax.inject.Inject

@Composable
fun HubScreen(
  navController: NavController,
  needContentPadding: Boolean = true,
  loader: suspend SpInternalApi.(SpApiManager) -> HubResponse,
  viewModel: HubScreenViewModel = hiltViewModel(),
  statusBarPadding: Boolean = false
) {
  val scope = rememberCoroutineScope()

  viewModel.needContentPadding = needContentPadding

  LaunchedEffect(Unit) {
    viewModel.load(loader)
  }

  when (viewModel.state) {
    is HubScreenViewModel.State.Loaded -> {
      LazyVerticalGrid(
        contentPadding = PaddingValues(if (needContentPadding) 16.dp else 0.dp),
        verticalArrangement = Arrangement.spacedBy(if (needContentPadding) 8.dp else 0.dp),
        horizontalArrangement = Arrangement.spacedBy(if (needContentPadding) 8.dp else 0.dp),
        columns = GridCells.Fixed(2),
        modifier = if (statusBarPadding) Modifier.statusBarsPadding() else Modifier
      ) {
        if (viewModel.needContentPadding) {
          item(span = { GridItemSpan(2) }) {
            Spacer(modifier = Modifier.statusBarsPadding())
          }
        }

        (viewModel.state as HubScreenViewModel.State.Loaded).data.apply {
          if (header != null) {
            item(
              key = header.id,
              span = {
                GridItemSpan(2)
              },
              contentType = header.component.javaClass.simpleName,
            ) {
              HubBinder(navController, viewModel, header)
            }
          }

          body.forEach { item ->
            if (item.component.isGrid() && !item.children.isNullOrEmpty()) {
              items(item.children, key = { dItem -> dItem.id }, contentType = {
                item.component.javaClass.simpleName
              }) { cItem ->
                HubBinder(navController, viewModel, cItem)
              }
            } else {
              item(span = {
                GridItemSpan(if (item.component.isGrid()) 1 else 2)
              }, key = item.id, contentType = {
                item.component.javaClass.simpleName
              }) {
                HubBinder(navController, viewModel, item)
              }
            }
          }
        }
      }
    }
    
    is HubScreenViewModel.State.Error -> PagingErrorPage(onReload = { scope.launch { viewModel.reload(loader) } }, modifier = Modifier.fillMaxSize())
    HubScreenViewModel.State.Loading -> PagingLoadingPage(modifier = Modifier.fillMaxSize())
  }
}

@HiltViewModel
class HubScreenViewModel @Inject constructor(
  private val spInternalApi: SpInternalApi,
  private val spApiManager: SpApiManager,
  private val spPlayerServiceManager: SpPlayerServiceManager
) : ViewModel(), HubScreenDelegate {
  private val _state = mutableStateOf<State>(State.Loading)
  val state: State get() = _state.value

  // no state handle needed
  var needContentPadding: Boolean = false

  suspend fun load(loader: suspend SpInternalApi.(SpApiManager) -> HubResponse) {
    _state.value = try {
      State.Loaded(spInternalApi.loader(spApiManager))
    } catch (e: Exception) {
      e.printStackTrace()
      State.Error(e)
    }
  }

  suspend fun reload(loader: suspend SpInternalApi.(SpApiManager) -> HubResponse) {
    _state.value = State.Loading
    load(loader)
  }

  override fun play(data: PlayFromContextData) {
    spPlayerServiceManager.play(data.uri, data.player)
  }

  override fun isSurroundedWithPadding() = needContentPadding

  override suspend fun calculateDominantColor(url: String, dark: Boolean): Color {
    return try {
      val apiResult = spApiManager.partners.getDominantColors(url).data.extractedColors[0].let {
        if (dark) it.colorRaw else it.colorDark
      }.hex

      Color(android.graphics.Color.parseColor(apiResult))
    } catch (e: Exception) {
      // e.printStackTrace()
      Color.Transparent
    }
  }

  sealed class State {
    class Loaded(val data: HubResponse) : State()
    class Error(val error: Exception) : State()
    object Loading : State()
  }
}